#include "stdafx.h"

using namespace std;

inline static float _fabs(float f){ return f<0?-f:f; }


void FU::randomize()				// Randomizes Functional Unit
{
	type=rand()%TYPECOUNT;
	input0=rand()%pos;
	input1=rand()%pos;
//	input2=rand()%pos;
}


void FU::mutate()
{
	int r=rand();
	
	switch(type){						// se til at bare brukte inputs eller type muteres

		case VCC:
		case GND:
			r=0;
			break;

		case NOT:
			r%=2;
			break;

		case NAND:
		case NOR:
		case AND:
		case OR:
			r%=3;
			break;

		default:
			FAULT_END("INVALID GATE TYPE FOR MUTATION");
	}

	switch(r){						// what to mutate
	case 0:							// mutate type
		type=rand()%TYPECOUNT;
		input1%=pos;				// garanter at input1 ikke er hyere enn pos... men kan den vre det???
		break;

	case 1:							// mutate input 0
		input0=rand()%pos;
		break;

	case 2:							// mutate input 1
		input1=rand()%pos;
		break;

//	case 3:							// mutate input 2
//		input2=rand()%pos;
//		break;

	default:
		FAULT_END("UNKNOWN MUTATION OCCURED");
	}	
}


// SIGMOID TABLE/FUNCTION GENERATION

//This method calculates a sigmoid
inline static float _sigmoid(float x, float a=15.0f){
	return (float)(1.0/(1.0+exp(-a*x)));
}

//this method linearly maps floats in the range [-2.0, 2.0] to ints in the range [0, 16384]
//this gives 4097 steps in the range [-.5, 0.5], and expands enough on both ends for our use
inline static int map(float x){
	return ((int)(4096.0f*x))+8192;
}

//this method calculates the lookup table
//notice how our entire table is 64kB, so it fits in most modern L1 caches
//and how we invert it since all our gates have inverted output
//Could also have done the translation by 0.5 here to save a few cycles
inline static float* makeTable(){
	float* tab=new float[16385];
	//for(int i=0;i<12289;++i){
	for(int i=0;i<16385;++i){
		tab[i]=_sigmoid( (i-8192)*(1/4096.0f) );
	}
	return tab;
}

//a lookup table for our sigmoid
static const float *table=makeTable();

//calculates a sigmoid by looking in a table
inline static float sigmoid(float x){
	if (x<-1)
		x = -1;		// we can do this will return lim->0 anyway
	if (x>1)
		x = 1;		// this will return lim->1 anyway
	return table[map(x)];
}

// END OF SIGMOID TABLE/FUNCTION GENERATION



float FU::evaluate(float* inputs, Parameters* params, bool fail)
{
	float A = inputs[input0];
	float B = inputs[input1];
	float C;

	// CHECKING FOR ERRORS (AND ACTIVATE ANY INPUT ERRORS)

	bool error_output=false;
	if (fail) {
		unsigned int sum = params->rel_both_fail + params->rel_in_fail + params->rel_out_fail;
		unsigned int f = rand() % sum;

    //find out if in or out or both fails
		if(f < params->rel_out_fail){//only out fails
			error_output=true;
		}else{//in fails, and maybe out
			error_output = (f < params->rel_out_fail + params->rel_both_fail);//test if out fails

			//find out which input fails
			sum = params->rel_ina_fail + params->rel_inb_fail + params->rel_in_both_fail;
			f = rand() % sum;

			//if input A fails
			if(f < params->rel_ina_fail+params->rel_in_both_fail){
				//simulate a stuck-at error
				A = rand() & 1;
			}

			//if input B fails
			if(f >= params->rel_ina_fail){
				//simulate a stuck-at error
				B = rand() & 1;
			}
		}
	}
	
	// CALCULATE FUNCTION OUTPUT

	switch(type)
	{

		case NOT:	C = sigmoid(0.5f-A); break;

		case AND:	C = sigmoid(A*B - 0.5f); break;

		case NAND:	C = sigmoid(-(A*B - 0.5f));	break;

		case OR:	C = sigmoid(A+B - A*B - 0.5f); break;

		case NOR:	C = sigmoid(-(A+B - A*B - 0.5f)); break;

		case VCC:	C = 1; break;

		case GND:	C = 0; break;

		default:
			FAULT_END("UNKNOWN GATE TYPE ACTIVATED!");

 	}
/* Old, pure logical evaluation
	switch(type)
	{
		case NOT:
			C = 1-A;
			break;

		case NAND:
			if (!(A==1 && B==1))
				C = 1;
			else
				C = 0;
			break;

		case NOR:
			if (!(A==1 || B==1))
				C = 1;
			else
				C = 0;
			break;

		case AND:
			if (A==1 && B==1)
				C = 1;
			else
				C = 0;
			break;

		case OR:
			if (A==1 || B==1)
				C = 1;
			else
				C = 0;
			break;

		case VCC:
			C = 1;
			break;

		case GND:
			C = 0;
			break;

		default:
			FAULT_END("UNKNOWN GATE TYPE ACTIVATED!");
	}
*/


// CHECKING FOR ERRORS (AND ACTIVATE ANY INPUT ERRORS)

	if(error_output){
    if (params->fail_type==1)
      C = 1 - C;         // invert output if fail_type=1
    else {
      switch(rand()%3){  // otherwise use oldschool 3 different faults:
        case 0:          // stuck-at
          C = rand() & 1;
          break;
        case 1:          // floating output
          C = rand() * (1.0f/RAND_MAX);
          break;
        case 2:          // partly random output (50% correct or random)
          {
            float errorlevel = _fabs( C - (rand() & 1) );
            float random = rand() * (1.0f/RAND_MAX);
            
            C = errorlevel * random + (1-errorlevel) * C;
          }
          break;
        default:
          throw "Shouldn't happen";
      }
    }
	}

// ADDING NOISE TO OUTPUT	

	if(params->noise)
		return C + params->noise*(rand()*(2.0f/RAND_MAX)-1.0f);
	else
		return C;

}

void FU::setUsed(bool* b, int& in0, int& in1)	// setter true i b-array dersom et input er i bruk
{
  in0 = -1; // -1 means dont parse further/dead end inputs
  in1 = -1;
  
	switch(type)
	{
		case VCC:
		case GND:
			break;

		case NOT:
			b[input0]=true;
      in0=input0;
			break;

		case NAND:
		case NOR:
		case AND:
		case OR:
			b[input0]=true;
			b[input1]=true;
      in0=input0;
      in1=input1;
			break;

		default:
			b[input0]=true;
			b[input1]=true;
      in0=input0;
      in1=input1;
			//b[input2]=true;
	}
}

unsigned char FU::isVCCGND() // true if gate is VCC or GND
{
  if ( (type==VCC) || (type==GND) )
    return 1;
  return 0;

}
  
int FU::getInput(unsigned char inputnumber)
{
  int input = -1; // -1 means dont parse further/dead end inputs
  
	switch(type)
	{
		case VCC:
		case GND:
			break;

		case NOT:
      if(inputnumber==0)
        input=input0;
			break;

		case NAND:
		case NOR:
		case AND:
		case OR:
      if(inputnumber==0)
        input=input0;
      else
        input=input1;
			break;
	}
  return input;
}


std::ostream& operator<<(std::ostream &os, const FU& fu)
{
	os << "(" << (unsigned int)fu.pos << ": ";

	switch(fu.type){
	case FU::VCC:
		os << "VCC ";
		break;
	case FU::GND:
		os << "GND ";
		break;

	case FU::NOT:
		os << "NOT " << (unsigned int)fu.input0;
		break;

	case FU::NAND:
		os << (unsigned int)fu.input0 << " NAND " << (unsigned int)fu.input1;
		break;
	case FU::NOR:
		os << (unsigned int)fu.input0 << " NOR " << (unsigned int)fu.input1;
		break;
	case FU::AND:
		os << (unsigned int)fu.input0 << " AND " << (unsigned int)fu.input1;
		break;
	case FU::OR:
		os << (unsigned int)fu.input0 << " OR " << (unsigned int)fu.input1;
		break;
/*
	case FU::MUX:
		os << (unsigned int)fu.input2 << " MUX " << (unsigned int)fu.input0 << " + " << (unsigned int)fu.input1;
		break;
	case FU::NOTMUX:
		os << (unsigned int)fu.input2 << " NOTMUX " << (unsigned int)fu.input0 << " + " << (unsigned int)fu.input1;
		break;
*/
	}

	os << ")";

	return os;
}

std::istream& operator>>(std::istream &is, FU& fu)
{

	char ctmp;
	int itmp;
	std::string stmp;

	is >> ctmp;
	if(ctmp!='(')
		FAULT_END("FU read error 1");

	is >> itmp;
	fu.pos=itmp;

	is >> ctmp;
	if(ctmp!=':')
		FAULT_END("FU read error 2");

	is >> stmp;
	
	if(stmp=="VCC"){
		fu.type=FU::VCC;
	}else if(stmp=="GND"){
		fu.type=FU::GND;
	
	}else if(stmp=="NOT"){
		fu.type=FU::NOT;
		is >> itmp;
		fu.input0=itmp;
	
	}else{
		fu.input0=atoi(stmp.c_str());

		is >> stmp;
		if(stmp=="NAND"){
			fu.type=FU::NAND;
			is >> itmp;
			fu.input1=itmp;
		}else if(stmp=="NOR"){
			fu.type=FU::NOR;
			is >> itmp;
			fu.input1=itmp;
		}else if(stmp=="OR"){
			fu.type=FU::OR;
			is >> itmp;
			fu.input1=itmp;
		}else if(stmp=="AND"){
			fu.type=FU::AND;
			is >> itmp;
			fu.input1=itmp;
		}
	}

	is >> ctmp;
	if(ctmp!=')')
		FAULT_END("FU read error 3");

	return is;
}
