#include "stdafx.h"

using namespace std;

extern unsigned int debug;

Generation::Generation(Parameters *params):								// Constructor for generation 0
iCount(params->individuals),
mostFit(-1)
{
	individuals=new Individual[iCount];
	
	if(params->individual){												// If an individual is specified, use it
		mostFit=0;
		for(int a=iCount-1;a>=0;--a)
			individuals[a]=*params->individual;
	}else{
		for(int a=iCount-1;a>=0;--a)
			individuals[a].init(params->gateCount, params->inputCount);
	}
}

// BETACODE FOR GENERATION IMPORT
// {
//   if (params->preloadgeneration==0){
//     individuals=new Individual[iCount];
    
//     if(params->generation){
//       if (params->generation->iCount > iCount)
//         cout << "\nWARNING: Specified generation larger than specified population size. Surplus individuals will be discarded.\n";
//       else if (iCount > params->generation->iCount)
//         cout << "\nWARNING: Specified generation smaller than specified population size. Surplus individuals will be randomized.\n";
      
//       mostFit=params->generation->mostFit;
      
//       for(int a=0;a<iCount;a++)
//         if (a>=params->generation->iCount)
//           individuals[a].init(params->gateCount, params->inputCount);
//         else
//           individuals[a]=params->generation->individuals[a];
      
//     }
    
//     else if(params->individual){												// If an individual is specified, use it
//       mostFit=0;
//       for(int a=iCount-1;a>=0;--a)
//         individuals[a]=*params->individual;
//     }
//     else{
//       for(int a=iCount-1;a>=0;--a)
//         individuals[a].init(params->gateCount, params->inputCount);
//     }
//   }
//   else //if preload
//     for(int a=iCount-1;a>=0;--a)
//       individuals[a].init(params->gateCount, params->inputCount);
// }

Generation::Generation(const Generation *oldgen, Parameters* params):	// Constructor for all but generation 0, using GA and prev. gen.
mostFit(-1)
{
	iCount=oldgen->size();
	individuals=new Individual[iCount];

	//use the specified GA
	switch(params->algorithm){
	case MUTATE_FITTEST_1:
		{
			//clone the most fit individual, and mutate all but one

			const Individual* i=oldgen->getMostFit();

			int a;

			for(a=iCount-1;a>=0;--a)
				individuals[a]=*i;

			for(a=iCount-1;a>0;--a)
				individuals[a].mutate(params->ga_mutationrate);

			break;
		}
	case MUTATE_FITTEST_2:
		FAULT_END("NOT YET IMPLEMENTED: GA MUTATE_FITTEST_2"); 

	case KILL_HALF:
		FAULT_END("NOT YET IMPLEMENTED: GA KILL_HALF");

	case KOOZA:
		FAULT_END("NOT YET IMPLEMENTED: GA KOZA");

	case TOURNAMENT:
		{
			const Individual* i=oldgen->getMostFit();									// i is the best individual of the old gen
			
			//cout << (*oldgen)[1].getFitness();
			//Ind=((*oldgen)[0])
			int g,p,c;																	// counts Group, Parents, Children
			int parents[2];																// parents, index to old gen
			float best_in_group_fitness;
			int newgen_index = 0;
			int *group_index;
			group_index = new int[params->ga_groupsize];								// array of group (index to old gen)

			if (!(params->ga_elitism==0)) {												// if ga_elitism, copy the best individual
				individuals[newgen_index++] = *i;
				if (debug & DBG_GA)
					cout << "Child " << "0 " << " is elite copy" << endl; //from " << oldgen->getMostFitNumber() << endl;
			}				

			for(c=newgen_index;c<iCount;c+=2) {											// FOR each new pair of children
				for(p=0;p<2;p++) {														// FOR each parent (of 2)
					best_in_group_fitness = -1e6;
					for(g=0;g<params->ga_groupsize;g++)									// randomly pick the members of the group
						group_index[g] = rand()%iCount;

					if (params->ga_selectionprobability < (rand()*(1.0f/RAND_MAX))){	// if just randomly select
						parents[p] = group_index[rand()%params->ga_groupsize];			// pick a random member of the group
			
					} else {															// otherwise find the best in the group
						for(g=0;g<params->ga_groupsize;g++){
							if( (*oldgen)[group_index[g]].getFitness() > best_in_group_fitness) {
								best_in_group_fitness = (*oldgen)[group_index[g]].getFitness();
								parents[p] = group_index[g];							// just debugged this from "=g" to "=group_index[g]"
							}
						}
					}			
				}
			
			if (params->ga_crossoverrate > (rand()*(1.0f/RAND_MAX))){					// if we are doing crossover
			
				unsigned int crosspoint = 1+(rand()%(params->gateCount-1));
				if (debug & DBG_GA)
					cout << "Child " << c << " and " << c+1 << " cross from " << parents[0] << " and " << parents[1] << " at point " << crosspoint << endl;
				individuals[c]   = (*oldgen)[parents[0]];
				individuals[c].cross((*oldgen)[parents[1]],crosspoint);

				if (!((c+1) == iCount)) {												// unless this is the one spare child
					individuals[c+1] = (*oldgen)[parents[1]];
					individuals[c+1].cross((*oldgen)[parents[0]],crosspoint);
				}

			} else {																	// if not cross, just copy

				individuals[c]   = (*oldgen)[parents[0]];
				if (debug & DBG_GA)
					cout << "Child " << c << " and " << c+1 << " copy from  " << parents[0] << " and " << parents[1] << endl;
				if (!((c+1) == iCount))													// unless this is the one spare child
					individuals[c+1] = (*oldgen)[parents[1]];
				}

			}
				
		for(c=newgen_index;c<iCount;c++) {												// mutate the children
			individuals[c].mutate(params->ga_mutationrate);}

			
		delete[] group_index;
		break;
		}
	
	default:
		cerr << endl << "Undefined GA: " << params->algorithm << endl;
		exit(1);
	}
}


Generation::~Generation()
{	delete[] individuals;}



void Generation::randomize()			//Randomizes all individuals in the generation.
{
	for(int a=iCount-1;a>=0;--a)
		individuals[a].randomize();
}

int Generation::getMostFitNumber()
{
	return mostFit;						// If evaluation has not been done
}

Individual* Generation::getMostFit()
{
	if(mostFit==-1)						// If evaluation has not been done
		return NULL;

	return &individuals[mostFit];
}


const Individual* Generation::getMostFit() const	// CONST version
{
	if(mostFit==-1)
		return NULL;

	return &individuals[mostFit];
}


void Generation::evaluate(Parameters* params)
{
	//Allocate these arrays here to avoid having to allocate them for each individual.
	//They are used in the evaluation.
	
	float*	data=	new float[params->inputCount+individuals[0].size()];
	bool*	used=	new bool[params->inputCount+individuals[0].size()];
	int*	usedbuf=new int[params->inputCount+individuals[0].size()];
	bool*	failed=	new bool[params->inputCount+individuals[0].size()];
	bool* 	evaluated=new bool[params->inputCount+individuals[0].size()];
  
	// Data:	Holds all block output values, starting with the inputs. In this array
	//			blockdata points to the first gate after the inputs and
	//			output points to the first of the output gates
	
	// Used:	An array of bools corresponding to the data array, with the value T/F whether that
	//			gate is used. Again, blockused is a shortcut to the first gate after the inputs.
	
	// Usedbuf:	An array of INT, where each specifies the number of a used gate, thus indexing
	//			the used array. E.g. usedbuf[0]==19 means blockused[19] is a used gate.

	// Failed:	An array of bools, corresponding to the numbering in usedbuf. E.g. if failed[18]==true
	//			means gate 18 failed, e.g. blockused[19] or blockdata[19]

  // Evaluated: Stores whether the block had been evaluated (similar to used)

	
	float max=-1e6;														// hold the lowest fitness value found so far, assume this is small enough...

	for(unsigned int a=0;a<iCount;++a){
		individuals[a].evaluate(params, data, used, usedbuf, failed, evaluated, (params->inputCount+individuals[0].size()));	// evaluate each individual

    if(individuals[a].getFitness()>=max){							// remember the best individual found so far
			max=individuals[a].getFitness();							// New; ">=" ensures neutrality (old was "=")
			mostFit=a;
		}
	}

	
	delete[] data;														// delete temporary arrays
	delete[] used;
	delete[] usedbuf;
	delete[] failed;
  delete[] evaluated;
}

std::ostream& operator<<(std::ostream &os, const Generation& g)
{

	os << "Generation[ popsize: " << g.iCount << " mostfit: "<< g.mostFit << " population: ";

	for(unsigned int a=0;a<g.iCount;++a) {
		os << endl << g.individuals[a];}

	os << " ]" << endl;
	return os;
}

std::istream& operator>>(std::istream &is, Generation& g)
{
	std::string stmp;	
	int itmp;
	char ctmp;
  int individuals;
  
  // så lenge pop ikke er full, fortsett å lese ind dersom de finnes
  // rapporter ulikhet i pop/#inn_ind, begge veier

  is >> stmp;
 	if(stmp!="Generation[")
 		FAULT_END("Generation Read error");

 	is >> stmp;
	if(stmp!="popsize:")
		FAULT_END("Generation Read error");

  is >> individuals;
  if (individuals > g.iCount){
    cout << "\nWARNING: Specified generation larger than specified population size. Surplus individuals will be discarded.\n";
    individuals = g.iCount;
  }
  else
    g.iCount = individuals;
  
 	is >> stmp;
	if(stmp!="mostfit:")
		FAULT_END("Generation Read error");

  is >> itmp;
  g.mostFit=itmp;

 	is >> stmp;
	if(stmp!="population:")
		FAULT_END("Generation Read error");

  for (int a=0;a<individuals;a++)
    is >> g.individuals[a];

  is >> ctmp;
  if(ctmp!=']')
    FAULT_END("Individual Read error");
  
  return is;
}
