#include "stdafx.h"
#include <string.h>

inline static float _fabs(float f){ return f<0?-f:f; }

using namespace std;

extern unsigned int debug;

// The created individual is not initialized, so
// you must call either init() or operator=() before it is used.

Individual::Individual():
blocks(0),
unitCount(0)
{}

Individual::~Individual()
{
	delete[] blocks;
}

void Individual::operator=(const Individual& i)			
// Make a copy of an individual
{
	if(blocks)
	// slette eventuelle FU
		delete[] blocks;

	// kopier antall FU, fitness og alle FUer
	unitCount=i.unitCount;	
	fitness=i.fitness;
	reliability=i.reliability;
	usedCount=i.usedCount;
	blocks=new FU[unitCount];
	for(unsigned int a=0;a<unitCount;++a)
		blocks[a]=i.blocks[a];
}

void Individual::cross(const Individual& mate, unsigned int crosspoint)
{
	for(unsigned int a=crosspoint;a<unitCount;a++)
		blocks[a]=mate.blocks[a];
}

void Individual::init(unsigned int count, int inCount)	// Initialize individual, allocate memory
{
	unitCount=count;
	blocks=new FU[count];

	for(unsigned int a=0;a<count;++a)
		blocks[a].setPos(a+inCount);
}

void Individual::randomize()	
// Randomize every FU
{
	for(unsigned int a=0;a<unitCount;++a)
		blocks[a].randomize();
}


void Individual::evaluate(Parameters *params, float* data, bool* used, int* usedbuf, bool* failed, bool* evaluated, int arraySize)
{
	//real gatecount does not count vcc/gnd as used
	unsigned int a, b, run, test, real_gatecount; 
	int in0, in1, top, top_short;
  
  float fitness_tmp;

	// Data:	Holds all block output values, starting with the inputs. In this array
	//		blockdata points to the first gate after the inputs and
	//		output points to the first of the output gates
	
	// Used:	An array of bools corresponding to the data array, with the value T/F whether that
	//		gate is used. Again, blockused is a shortcut to the first gate after the inputs.
	
	// Usedbuf:	An array of INT, where each specifies the number of a used gate, thus indexing
	//		the used array. E.g. usedbuf[0]==19 means blockused[19] is a used gate.

	// Failed:	An array of bools, corresponding to the numbering in usedbuf. E.g. if failed[18]==true
	//		means gate 18 failed, e.g. blockused[19] or blockdata[19]

	// Number of units+inputs (length of all arrays)
	unsigned int dataLength=params->inputCount+unitCount;
	// Shortcut to the output of the FUs
	float* blockdata=&data[params->inputCount];
	// Shortcut to the output of the individual	
	float* output=&blockdata[unitCount-params->outputCount];
	// Shortcut to the used-state of the FUs
	bool* blockused=&used[params->inputCount];
	stack<int> Stack;

	// Reset fitness value
	fitness=0;
	fitness_tmp=0;
	reliability=0;

	// Set all FUs as not used
	for(a=0;a<dataLength;++a)
		used[a]=false;

	// Set output FUs as used - they are always used
	for(a=1;a<=params->outputCount;++a) {
		blockused[unitCount-a]=true;
		//cout << "blockused[" << unitCount-a << "] is true\n";
		Stack.push(unitCount-a);
	}

	while( !Stack.empty() ) {   // new stack-loop for detecting used gates in non-CGP networks
		blocks[Stack.top()].setUsed(used,in0,in1);
		in0-=params->inputCount;
		in1-=params->inputCount;
		Stack.pop();
		if (in0>=0)
			Stack.push(in0);
		if (in1>=0)
			Stack.push(in1);
	}

	usedCount=0;
	real_gatecount=0;

	// For each FU, update usedbuf and usedCount after stack-loop
	for(a=unitCount-1;a<unitCount;--a){	
		if(blockused[a]){
			usedbuf[usedCount]=a;
			usedCount++;
			if(!blocks[a].isVCCGND())
				real_gatecount++;
		}
	}

  
	// OLD used-setting loop (only stricly feedforward CGP networks)
  /*	usedCount=0;
	
	// For each FU: if it is used set its inputs as used and increase usecount
	for(a=unitCount-1;a<unitCount;--a){
		// Continue backwards through the network
		if(blockused[a]){
			// The unsigned value of a wraps around to end the FOR loop
			blocks[a].setUsed(used,in0,in1);
			cout << endl << "block " << a << " in0:" << in0 << " in1:" << in1;
			usedbuf[usedCount]=a;
			++usedCount;
		}
    }*/



//******************************************************
//  Start of Stuck-at kode
//******************************************************
	unsigned int temp = params->number_of_ssa;



	if (params->single_stuck_at){
		// if the number of stuck at point is bigger den the 
		// avaleble blocks used, sett it to stuck at all (=0)
		if (params->number_of_ssa >= usedCount || (params->output_protect && params->number_of_ssa >= usedCount-params->outputCount))
			temp = 0;

		if (temp)
			// For a given number of stuck at points
			params->error_runs = params->number_of_ssa;
		else if (params->output_protect)
			// Stuck at all, with output protection
			params->error_runs = usedCount - params->outputCount;
		else
			// Stuck at all, without output protection
			params->error_runs = usedCount;	
	}//end if

	int* stuck_at = new int[params->error_runs];

	for (int t = 0 ; t < params->error_runs; t++)
		stuck_at[t] = -1;

	// Generate whitch gates to stick at
	if (temp){				
		unsigned int i = 0;
		unsigned int j = 0;
		unsigned int number;

		// while the array is not full
		while(i < temp){	
			if (params->output_protect)
				// generates a number between outputCount and usedCount
				number = ((rand()%(usedCount-params->outputCount))+params->outputCount); 
			else
				// generates a number between 0 and usedCount
				number = rand()%usedCount;

			j = 0;

			//checks the array to see if this block is added already
			while(stuck_at[j] != -1){
				if (stuck_at[j] == usedbuf[number])
					break;
				else 
					j++;
			}//end while
			// if it is'nt in the array
			if (stuck_at[j] == -1){	
				stuck_at[j] = usedbuf[number];// add it
				i++;	
			}//end if
		}//end while
	} else
		for (int w = 0; w < params->error_runs; w++)
			stuck_at[w] = usedbuf[w];

//******************************************************
//  End of Stuck-at preparation
//******************************************************


	// For each error configuration
	for(run=0;run<params->error_runs;++run){

		for(a=0;a<unitCount;a++)
		  // initially none fail
		  failed[a] = false;

		// start checking for fail at first gate in usedbuf
		unsigned int failstart=0;
		if (params->output_protect)
		  // start after output count if output gates are protected
		  failstart=params->outputCount;

		if (params->failrate) {
		  // check for failures
		  for(a=failstart;a<usedCount;a++)
		    if ( (rand()*(1.0f/RAND_MAX)) > params->failrate)
		      failed[usedbuf[a]]=false;
		    else
		      failed[usedbuf[a]]=true;
		} else if (params->single_stuck_at){
		  // removes the faults for last run
		  for(int t = 0; t < arraySize;t++) 
		    failed[t] = false;

		  failed[stuck_at[run]]=true;

		}//end if
    
		for(test=0; test < params->error_tests; ++test){	// For each test run
		  for(a=0;a<params->dataCount;++a){			// For each test vector
		    for(b=0;b<params->inputCount;++b)			// Copy inputs into data array
			    data[b]=params->inputs[a][b];

		    // The inputs values are allready set / evaluated
		    for(b=0;b<params->inputCount;b++)   
		      evaluated[b]=true;

		    // Others are not yet evaluated
		    for(b=params->inputCount;b<dataLength;b++)
		      evaluated[b]=false;

		    // Prepare outputs for evaluation and parse network
		    for(b=1;b<=params->outputCount;++b) 
		      // Use full index (not shortcut) on stack
		      Stack.push(unitCount-b+params->inputCount);

		    // LOOP FOR EVALUATION (supports non-strict feed forward)

		    while( !Stack.empty() ) {

		      top = Stack.top();
		      top_short = Stack.top()-params->inputCount;

		      // if this FU has allready been eval'ed, pop it
		      if( evaluated[top] )
			Stack.pop();

		      else {
			in0=blocks[top_short].getInput(0);
			in1=blocks[top_short].getInput(1);

		        if(in0 == -1) { // *** zero inputs ***
			  blockdata[top_short]=blocks[top_short].evaluate(data, params, failed[top_short]); // eval'
			  evaluated[top]=true;
				  Stack.pop();
			} else if(in1 == -1) // *** single input ***
			  if(evaluated[in0]){ // if that input is eval'ed
			    blockdata[top_short]=blocks[top_short].evaluate(data, params, failed[top_short]); // eval'
			    evaluated[top]=true;
			    Stack.pop();
			  } else
			    Stack.push(in0); // else push it

			// *** two inputs ***
			else {

			  // if first input not eval'ed
			  if(!evaluated[in0])
			    Stack.push(in0);   // push it

			  // if second input not eval'ed
			  if(!evaluated[in1])
			    Stack.push(in1);   // push it

			  else if((evaluated[in0])&&(evaluated[in1])) {
			    blockdata[top_short]=blocks[top_short].evaluate(data, params, failed[top_short]); // eval'
			    evaluated[top]=true;
			    Stack.pop();
 
			  }//end if
			}//end if
		      }//end if
		    }//end while

		    // Compute fitness as negative sum of deviation from target
		    for(b=0;b<params->outputCount;++b) {
		      fitness_tmp-=_fabs(params->outputs[a][b]- (output[b]>=0.5f?1.0f:0.0f));

		      if (debug & DBG_EVALUATION){
		      	cout << "output [" << b << "] is " << output[b];
		      	cout << ", correct is " << params->outputs[a][b] << endl;
		      }//end if
		    }//end for
		  } // end of test vector

		  if (fitness_tmp==0)
		    // if all vectors correct (correct circuit), increase realiability
		    reliability++;

		  fitness+=fitness_tmp;
		  fitness_tmp=0;
		} //end of test run

	} // end of error configuration

	// Average fitness
	fitness/=(params->error_runs*params->error_tests);

	// Average reliability
	reliability/=(params->error_runs*params->error_tests);
  
	// fitness_method==1 calculates fitness as realiability (1->good, 0->faulty)
	if (params->fitness_method == 1) 
		fitness = reliability;

	// START OF OPTIONAL NONFAULTED TEST
	if (params->fitness_method == 2) {  // combine truthtable fitness and reliability
	// give dominating score to those whose unfaulted function is 100% correct
		fitness_tmp=0;
    
		for(a=0;a<unitCount;a++)
		// initially none fail
		failed[a] = false;		

		// For each test vector
		for(a=0;a<params->dataCount;++a){
		  // Copy inputs into data array
		  for(b=0;b<params->inputCount;++b)
		    data[b]=params->inputs[a][b];

		  // The inputs values are allready set / evaluated
		  for(b=0;b<params->inputCount;b++)
		    evaluated[b]=true;

		  // Others are not yet evaluated
		  for(b=params->inputCount;b<dataLength;b++) 
		    evaluated[b]=false;

		  // Prepare outputs for evaluation and parse network
		  for(b=1;b<=params->outputCount;++b) 
		    // Use full index (not shortcut) on stack
		    Stack.push(unitCount-b+params->inputCount); 

		  while( !Stack.empty() ) {
		    top = Stack.top();
		    top_short = Stack.top()-params->inputCount;

		    // if this FU has allready been eval'ed, pop it
		    if( evaluated[top] ) 
		      Stack.pop();
		    else {
		      in0=blocks[top_short].getInput(0);
		      in1=blocks[top_short].getInput(1);
        	      if(in0 == -1) { // *** zero inputs ***
        		blockdata[top_short]=blocks[top_short].evaluate(data, params, failed[top_short]); // eval'
			evaluated[top]=true;
			Stack.pop();
		      } else if(in1 == -1) // *** single input ***

			// if that input is eval'ed
			if(evaluated[in0]){ 
			  blockdata[top_short]=blocks[top_short].evaluate(data, params, failed[top_short]); // eval'
			  evaluated[top]=true;
			  Stack.pop();
			} else
			  Stack.push(in0); // else push it
		      else {               // *** two inputs **

			// if first input not eval'e
			if(!evaluated[in0])
			  Stack.push(in0);   // push it

			// if second input not eval'ed
			if(!evaluated[in1])
			  Stack.push(in1);   // push it

			else if((evaluated[in0])&&(evaluated[in1])) {
			  blockdata[top_short]=blocks[top_short].evaluate(data, params, failed[top_short]); // eval'
			  evaluated[top]=true;
			  Stack.pop();
			}//end if
		      }//end if
		    }//end if
		  }//end while

		  // Compute fitness as negative sum of deviation from target       
		  for(b=0;b<params->outputCount;++b) {
		    fitness_tmp-=_fabs(params->outputs[a][b]- (output[b]>=0.5f?1.0f:0.0f));
		    if (debug & DBG_EVALUATION)
		      cout << "output [" << b << "] is " << output[b] << ", correct is " << params->outputs[a][b] << endl;
		  }
		} // end of test vector

		// if unfaulted fitness is MAX
		if (fitness_tmp==(params->dataCount*params->outputCount)) 
		  fitness+=fitness_tmp;  // add MAX to faulted fitness

	} // END OF EXTRA OPTIONAL NONFAULTED TEST

 
  
	//if(fitness>-1)
	//	fitness-=params->sizeFactor*usedCount;	// Add size value

	if (!params->count_vcc_gnd)
		usedCount=real_gatecount;
  
	delete[] stuck_at;
}


// Mutates FUs according to the given mutation rate
void Individual::mutate(float mutationrate)
{
	// new method, check prob. with every gate (slower)
	for (int a=0;a<unitCount;a++)
		if ((rand()*(1.0f/RAND_MAX))<mutationrate)
			blocks[a].mutate();
} 


std::ostream& operator<<(std::ostream &os, const Individual& i)
{
	os << "Individual{ size: " << i.unitCount;
	os << " used: " << i.usedCount;
	os << " fitness: " << i.fitness;
	os << " reliability: " << i.reliability << " blocks: ";

	for(unsigned int a=0;a<i.unitCount;++a){
		os << i.blocks[a];
	}

	os << " }";

	return os;
}
  
std::istream& operator>>(std::istream &is, Individual& i)
{
	std::string stmp;	
	int itmp;
	char ctmp;

	is >> stmp;
	if(stmp!="Individual{")
		FAULT_END("Individual Read error (Individual{)");

	is >> stmp;
	if(stmp!="size:")
		FAULT_END("Individual Read error (size:)");


	is >> itmp;
	i.init(itmp, 0);

	is >> stmp;
	if(stmp!="used:")
		FAULT_END("Individual Read error (used:)");
	
	is >> i.usedCount;

	is >> stmp;
	if(stmp!="fitness:")
		FAULT_END("Individual Read error (fitness:)");

	is >> i.fitness;

	is >> stmp;
	if(stmp!="reliability:")
		FAULT_END("Individual Read error (reliability:)");

	is >> i.reliability;

	is >> stmp;
	if(stmp!="blocks:")
		FAULT_END("Individual Read error (blocks:)");

	for(unsigned int a=0;a<i.unitCount;++a)
		is >> i.blocks[a];

	is >> ctmp;
	if(ctmp!='}')
		FAULT_END("Individual Read error (})");
	
	return is;
}
