

class Individual;

class Generation;

class Parameters{
public:
	unsigned int gateCount;			// number of gates (FUs) in one individual
	unsigned long generations;		// number of generations before evolution halts
	unsigned int individuals;		// number of individuals in each generation
	unsigned int mostFit;			// most fit individual if a full generation in given
	unsigned int reportGap;			// interval for reports
	int algorithm;				// identifies EA in use
//	float mutationRate;			// mutation rate used
	float sizeFactor;			// assign portion of fitness to small circuits
	float fitnessLimit;			// EA will halt when this limit is reached
	float bias;				// Bias used in Koza EA
	unsigned int stepwise_fitness;		// if 1, enable stepwise reliability/truthtable fitness function

	int ga_groupsize;			// GA Groupsize in Tournament Selection
	int ga_elitism;				// Number of elite individuals to clone to next generation (0=off)
	float ga_selectionprobability;		// GA Tournament - Probability of selecting best in group
	float ga_crossoverrate;			// Probability of crossover
	float ga_mutationrate;			// Probability of mutation
  
	unsigned long seed;			// stores random seed

	float noise;				// noise used
	float failrate;				// fail probability used
	unsigned int minfail;			// minimum number of failing gates
	unsigned int maxfail;			// maximum number of failing gates
	unsigned int error_tests;		// how many tests pr. individual (TpI)
	unsigned int error_runs; 
	unsigned int fitness_method;		// method for calculating fitness (default=0=oldschool, 1=realiability, 2=combo)
	unsigned int output_protect;		// protect output gates from failure (default=1/yes)
	unsigned int fail_type;     		// type of failure in gates (default=0,oldschool, 1=invert)
	unsigned int testrun;			// if this is a testrun, only test, not evolve
	unsigned int preloadgeneration; 	// tells whether to next generation init should be for preload
	unsigned int count_vcc_gnd; 		// whether to count vcc/gnd as used gates

	unsigned int rel_out_fail;		// probability distribution of failures
	unsigned int rel_in_fail;
	unsigned int rel_both_fail;
	unsigned int rel_ina_fail;
	unsigned int rel_inb_fail;
	unsigned int rel_in_both_fail;

	bool single_stuck_at;			// is this a single stuck at run?
	unsigned int number_of_ssa;		// how many stuck at points to check (0 = all)

	unsigned int inputCount;		// number of circuit inputs
	unsigned int outputCount;		// number of circuit outputs
	unsigned int dataCount;			// number of lines in truthtable (number of testvectors)
	
	unsigned int circuit_type;		// arbitrary string that identifies the target circuit

	float** inputs;				// input vectors (truthtable)
	float** outputs;			// output vectors

	Individual* individual;

	Generation* generation;
  
	Parameters(){ 
		memset(this, 0, sizeof(*this)); // allocates memory
		bias=1.0f;			// sets bias (force float)
		seed=time(NULL);		// generates random seed from time

		maxfail=UINT_MAX;		// sets maximum number of FUs to fail
		error_tests=1;
		error_runs=1;
		testrun=0;
		fitness_method=0;   		// default to oldschool fitness method
		fail_type=0;        		// default to type 0
		output_protect=1;   		// protect output by default
		preloadgeneration=0;
		count_vcc_gnd=0;    		// do not count vcc/gnd as used gates by default
	}//end Parameters()
};//end class
