# Diverse funksjoner for MAPDL 2006

#plukker ut #used av de med best fitness for hver fail-verdi
bestused <- function(data) {
  used <- c()
  fitness <- c()
  failvalues <- unique(data$t_fail)
  for(f in failvalues) {
    fitnessv <- data$t_fitness[data$t_fail==f]
    usedv <-data$used[data$t_fail==f]
    
    maxindex <- which.max(fitnessv)
    fitness <- c(fitness, fitnessv[maxindex])
    used <- c(used, usedv[maxindex])    
  }
  list(t_fail=failvalues, t_fitness=fitness, used=used)  
}

bestusedbyRehw <- function(data) {
  used <- c()
  fitness <- c()
  failvalues <- unique(data$t_fail)
  for(f in failvalues) {
    fitnessv <- data$t_fitness[data$t_fail==f]
    usedv <-data$used[data$t_fail==f]
    
    maxindex <- which.max(fitnessv)
    fitness <- c(fitness, fitnessv[maxindex])
    used <- c(used, usedv[maxindex])    
  }
  list(t_fail=failvalues, t_fitness=fitness, used=used)  
}

bestusedbyRtrad <- function(data) {
  used <- c()
  reliability <- c()
  failvalues <- unique(data$t_fail)
  for(f in failvalues) {
    fitnessv <- data$t_reliability[data$t_fail==f]
    usedv <-data$used[data$t_fail==f]
    
    maxindex <- which.max(fitnessv)
    reliability <- c(reliability, fitnessv[maxindex])
    used <- c(used, usedv[maxindex])    
  }
  list(t_fail=failvalues, t_reliability=reliability, used=used)  
}

makeboxplot2 <- function(data) {
  normal   <- data$lzc3b[data$fail==0]
  tolerant <- data$lzc3b[data$fail>0.10]
  boxplot(normal, tolerant)
}

avgplot <- function(datax,datay,xlabel="",ylabel="") {
  #stoy <- 2*runif(length(data$fail))
  #stoy2 <- 0.001*runif(length(data$fail))  
  #plot(data$fail+stoy2, data$lzc3b+stoy,pch=".")
  #pch="+"
  plot(datax, datay,ylab=ylabel,xlab=xlabel)  
  #lines(lowess(datax,datay),col="gray")
  fvals <- sort(unique(datax))
  avgfun <- function(x) mean(datay[datax==x])
  lines(fvals, lapply(fvals,avgfun),col="black")
}

avgline <- function(datax,datay) {
  fvals <- sort(unique(datax))
  avgfun <- function(x) mean(datay[datax==x])
  lines(fvals, lapply(fvals,avgfun),col="black")
}

avgline2 <- function(datax,datay,dcol,dpch) {
  fvals <- sort(unique(datax))
  avgfun <- function(x) mean(datay[datax==x])
  lines(fvals, lapply(fvals,avgfun),col=dcol,pch=dpch)
}

maxline <- function(datax,datay,dcol,dpch) {
  fvals <- sort(unique(datax))
  avgfun <- function(x) max(datay[datax==x])
  lines(fvals, lapply(fvals,avgfun),col=dcol,pch=dpch)
}

maxpoints <- function(datax,datay,dcol,dpch) {
  fvals <- sort(unique(datax))
  avgfun <- function(x) max(datay[datax==x])
  points(fvals, lapply(fvals,avgfun),col=dcol,pch=dpch)
}
minpoints <- function(datax,datay,dcol,dpch) {
  fvals <- sort(unique(datax))
  avgfun <- function(x) min(datay[datax==x])
  points(fvals, lapply(fvals,avgfun),col=dcol,pch=dpch)
}


makebarplot <- function(randdata, randgdata, evdata, from=0, to=1000, autorange=F, xlabel="") {
  if( autorange ) {
    from=min(c(randdata, randgdata, evdata))
    to=max(c(randdata, randgdata, evdata))
  }
  m <- makematrix(randdata, randgdata, evdata, from, to)
  maxy <- max(m)
  len <- dim(m)[2]

  barplot(m,names.arg=seq(from,to,by=8)[1:len],beside=TRUE,
          col=c("white","grey40","black"),ylim=c(0,maxy),xlab=xlabel,ylab="Distribution")
  legend(0, maxy, legend=c("random bits","random genotypes","evolved genotypes"),
         fill=c("white","grey40","black"))

}

makematrix <- function(randdata, randgdata, evdata, from=0, to=1000) {
    randhist  <- hist(randdata,  plot=FALSE, breaks=seq(from,to,by=8))$counts/length(randdata)
    randghist <- hist(randgdata, plot=FALSE, breaks=seq(from,to,by=8))$counts/length(randgdata)
    evhist    <- hist(evdata,    plot=FALSE, breaks=seq(from,to,by=8))$counts/length(evdata)

    matrix(c(randhist,randghist,evhist), nrow=3, ncol=(to-from)/8, byrow=TRUE)
}

examplesimple <- function() {
  values <- seq(0,2*pi, by=0.1)
  x <- sin(values)
  y <- cos(values)
  z <- sin(2*values)
  m <- matrix(c(x,y,z), nrow=3, ncol=length(values), byrow=T)
  barplot(m,beside=T,names.arg=values, ylim=c(-1.5,1.5),
          col=c("red", "green", "blue"),
          legend=c("sin(x)","cos(x)","sin(2*x)"))
}

makeboxplot <- function(rand1,rand2,randg1,randg2,randg3,ev1,ev2,ev3) {
  boxplot(rand1,rand2,randg1,randg2,randg3,ev1,ev2,ev3, notch=F,
          names=c("rnd504","rnd664","rnd_SF","rnd_1H","rnd_U2Z","ev_SF","ev_1H","evU2Z"),
          col="gray",ylab="LZ Complexity")
}





# makeplot <- function(randdata, randgdata, evdata, from=0, to=1000, autorange=F) {
#   if( autorange ) {
#     from=min(c(randdata, randgdata, evdata))
#     to=max(c(randdata, randgdata, evdata))
#   }
#   m <- makematrix(randdata, randgdata, evdata, from, to)
#   len <- dim(m)[2]
#   barplot(m,names.arg=seq(from,to,by=8)[1:len],beside=TRUE,
#           col=c("red","green","blue"))
#   legend(0.1,1,c("a","b","c"))

# }
